<?php
/**
 * @brief		Achievement Action Extension
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @subpackage	Invite System
 * @since		30 Mar 2022
 */

namespace IPS\invite\extensions\core\AchievementAction;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Achievement Action Extension
 */
class _invitation extends \IPS\core\Achievements\Actions\AbstractAchievementAction // NOTE: Other classes exist to provided bases for common situations, like where node-based filters will be required
{	
	/**
	 * Get filter form elements
	 *
	 * @param	array|NULL		$filters	Current filter values (if editing)
	 * @param	\IPS\Http\Url	$url		The URL the form is being shown on
	 * @return	array
	 */
	public function filters( ?array $filters, \IPS\Http\Url $url ): array
	{
		$return = array();

		$return['milestone'] = new \IPS\Helpers\Form\Number( 'achievement_filter_Invitation_nth', ( $filters AND isset( $filters['milestone'] ) AND $filters['milestone'] ) ? $filters['milestone'] : 0, FALSE, array(), NULL, \IPS\Member::loggedIn()->language()->addToStack('achievement_filter_nth_their'), \IPS\Member::loggedIn()->language()->addToStack('achievement_filter_invitation_nth_suffix') );
		$return['milestone']->label = \IPS\Member::loggedIn()->language()->addToStack('achievement_filter_NewContentItem_nth');

		return $return;
	}

	/**
	 * Format filter form values
	 *
	 * @param	array	$values	The values from the form
	 * @return	array
	 */
	public function formatFilterValues( array $values ): array
	{
		$return = array();

		if( isset( $values['achievement_filter_Invitation_nth'] ) )
		{
			$return['milestone'] = $values['achievement_filter_Invitation_nth'];
		}

		return $return;
	}

	/**
	 * Work out if the filters applies for a given action
	 *
	 * Important note for milestones: consider the context. This method is called by \IPS\Member::achievementAction(). If your code 
	 * calls that BEFORE making its change in the database (or there is read/write separation), you will need to add
	 * 1 to the value being considered for milestones
	 *
	 * @param	\IPS\Member	$subject	The subject member
	 * @param	array		$filters	The value returned by formatFilterValues()
	 * @param	mixed		$extra		Any additional information about what is happening (e.g. if a post is being made: the post object)
	 * @return	bool
	 */
	public function filtersMatch( \IPS\Member $subject, array $filters, $extra = NULL ): bool
	{
		if( isset( $filters['milestone'] ) )
		{
			if( \IPS\Db::i()->select( 'COUNT(*)', 'invite_invites', array( 'invite_sender_id=?', $extra['user']->member_id ) )->first() <> $filters['milestone'] )
			{
				return FALSE;
			}
		}

		return TRUE;
	}

	/**
     * Get the labels for the people this action might give awards to
     *
     * @param	array|NULL		$filters	Current filter values
     *
     * @return	array
     */
	public function awardOptions( ?array $filters ): array
	{
		return array(
			'subject' => 'achievement_filter_inviter',
		);
	}

	/**
	 * Get identifier to prevent the member being awarded points for the same action twice
	 * Must be unique within within of this domain, must not exceed 32 chars.
	 *
	 * @param	\IPS\Member	$subject	The subject member
	 * @param	mixed		$extra		Any additional information about what is happening (e.g. if a post is being made: the post object)
	 * @return	string
	 */
	public function identifier( \IPS\Member $subject, $extra = NULL ): string
	{
		return $subject->member_id;
	}

	/**
	 * Return a description for this action to show in the log
	 *
	 * @param	string	$identifier	The identifier as returned by identifier()
	 * @param	array	$actor		If the member was the "subject", "other", or both
	 * @return	string
	 */
	public function logRow( string $identifier, array $actor ): string
	{
		$member 	= \IPS\Member::load( $identifier );
		$memberLink = \IPS\Theme::i()->getTemplate( 'global', 'core', 'global' )->basicUrl( $member->url(), TRUE, $member->name, FALSE );

		return \IPS\Member::loggedIn()->language()->addToStack( 'AchievementAction__invitation_log', FALSE, array( 'htmlsprintf' => array( $memberLink ) ) );
	}
	
	/**
	 * Get "description" for rule
	 *
	 * @param	\IPS\core\Achievements\Rule	$rule	The rule
	 * @return	string|NULL
	 */
	public function ruleDescription( \IPS\core\Achievements\Rule $rule ): ?string
	{
		$conditions = array();

		if( isset( $rule->filters['milestone'] ) )
		{
			$conditions[] = \IPS\Member::loggedIn()->language()->addToStack( 'achievements_title_filter_invitation_milestone', FALSE, array(
				'htmlsprintf' => array(
					\IPS\Theme::i()->getTemplate( 'achievements' )->ruleDescriptionBadge( 'milestone', \IPS\Member::loggedIn()->language()->addToStack( 'achievements_title_filter_milestone_nth', FALSE, array( 'pluralize' => array( $rule->filters['milestone'] ) ) ) )
				),
				'sprintf' => array( \IPS\Member::loggedIn()->language()->addToStack('achievement_filter_invitation_nth_suffix') )
			) );
		}

		return \IPS\Theme::i()->getTemplate( 'achievements' )->ruleDescription( \IPS\Member::loggedIn()->language()->addToStack( 'AchievementAction__invitation_description_prefix' ), $conditions );
	}

	/**
	 * Get rebuild data
	 *
	 * @return	array
	 */
	static public function rebuildData()
	{
		return array( array(
			'table' => 'invite_invites',
			'pkey'  => 'invite_id',
			'date'  => 'invite_date',
			'where' => array(),
		) );
	}

	/**
	 * Process the rebuild row
	 *
	 * @param array		$row	Row from database
	 * @param array		$data	Data collected when starting rebuild [table, pkey...]
	 * @return void
	 */
	public static function rebuildRow( $row, $data )
	{	
		\IPS\Member::load( $row['invite_sender_id'] )->achievementAction( 'invite', 'invitation', \IPS\Member::load( $row['invite_conv_member_id'] ) );
	}
}