<?php

namespace IPS\invite;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Category Node
 */
class _Campaign extends \IPS\Node\Model
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'invite_campaigns';

	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'campaign_';

	/**
	 * @brief	Form in a popup
	 */
	public static $modalForms = TRUE;

	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'menu__invite_invite_campaign';

	/**
	 * @brief	[Node] ACP Restrictions
	 * @code
	 	array(
	 		'app'		=> 'core',				// The application key which holds the restrictrions
	 		'module'	=> 'foo',				// The module key which holds the restrictions
	 		'map'		=> array(				// [Optional] The key for each restriction - can alternatively use "prefix"
	 			'add'			=> 'foo_add',
	 			'edit'			=> 'foo_edit',
	 			'permissions'	=> 'foo_perms',
	 			'delete'		=> 'foo_delete'
	 		),
	 		'all'		=> 'foo_manage',		// [Optional] The key to use for any restriction not provided in the map (only needed if not providing all 4)
	 		'prefix'	=> 'foo_',				// [Optional] Rather than specifying each  key in the map, you can specify a prefix, and it will automatically look for restrictions with the key "[prefix]_add/edit/permissions/delete"
	 * @endcode
	 */
	protected static $restrictions = array(
		'app'		=> 'invite',
		'module'	=> 'invite',
		'prefix' 	=> 'campaign_'
	);

	/**
	 * @brief	[Node] App for permission index
	 */
	public static $permApp = 'invite';

	/**
	 * @brief	[Node] Title prefix.  If specified, will look for a language key with "{$key}_title" as the key
	 */
	public static $titleLangPrefix = 'invite_campaign_';

	/**
	 * @brief	[Node] Moderator Permission
	 */
	public static $modPerm = 'invite_campaign';

	/**
	 * @brief	[Node] Enabled/Disabled Column
	 */
	public static $databaseColumnEnabledDisabled = 'open';

	/**
	 * [Node] Get description
	 *
	 * @return	string
	 */
	protected function get__description()
	{
		return \IPS\Member::loggedIn()->language()->addToStack('campaign_code') . ': <strong>' . $this->code . '</strong>';
	}

	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		$form->add( new \IPS\Helpers\Form\Translatable( 'campaign_title', NULL, TRUE, array( 'app' => 'invite', 'key' => ( $this->id ? "invite_campaign_{$this->id}" : NULL ) ) ) );		
		$form->add( new \IPS\Helpers\Form\Number( 'campaign_duration', $this->id ? $this->duration : 30, FALSE, array( 'min' => 1, 'max' => 999 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack( 'days' ), 'campaign_duration' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'campaign_max_new_members', $this->id ? $this->max_new_members : -1, FALSE, array( 'unlimited' => -1, 'unlimitedLang' => 'no_restriction' ), NULL, NULL, NULL, 'campaign_max_new_members' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'campaign_open', $this->id ? $this->open : TRUE, FALSE, array(), NULL, NULL, NULL, 'campaign_open' ) );
	}

	/**
	 * [Node] Format form values from add/edit form for save
	 *
	 * @param	array	$values	Values from the form
	 * @return	array
	 */
	public function formatFormValues( $values )
	{
		if( !$this->id )
		{
			$this->save();			
		}

		\IPS\Lang::saveCustom( 'invite', "invite_campaign_{$this->id}", $values['campaign_title'] );
		unset( $values['campaign_title'] );

		if( !\IPS\Request::i()->id )
		{
			$this->code = static::generateRandomString();
		}

		return $values;
	}

	public function generateRandomString()
	{
		$length = \IPS\Settings::i()->is_campaign_code_len;

		switch( \IPS\Settings::i()->is_campaign_code_type )
		{
			case 'numbers':
				$characters = '0123456789';
			break;
			case 'letters':
				$characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
			break;
			case 'both':
				$characters = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
			break;
		}

		$stringLength = mb_strlen( $characters );
		$string = '';

		for( $i = 0; $i < $length; $i++ )
		{
			$string .= $characters[ rand( 0, $stringLength - 1 ) ];
		}

		return $string;
	}

	public static function getCampaignByCode( $code )
	{
		try
		{
			$campaign = \IPS\invite\Campaign::constructFromData( \IPS\Db::i()->select( '*', 'invite_campaigns',array( 'campaign_code=?', $code ) )->first() );
		}
		catch ( \UnderflowException $e )
		{
			$campaign = NULL;
		}

		return $campaign;
	}

	public function getNumberOfMembersInCampaign()
	{
		return $this->converted_members;
	}

	public static function canUseCampaign()
	{
		if( \IPS\Settings::i()->is_campaign_on )
		{
			if( \count( \IPS\invite\Campaign::roots() ) )
			{
				foreach( \IPS\invite\Campaign::roots() as $campaign )
				{
					if( $campaign->open )
					{
						return TRUE;
						break;
					}
				}
			}
		}

		return FALSE;
	}

	/**
	 * [Node] Get buttons to display in tree
	 * Example code explains return value
	 *
	 * @code
	array(
	array(
	'icon'	=>	'plus-circle', // Name of FontAwesome icon to use
	'title'	=> 'foo',		// Language key to use for button's title parameter
	'link'	=> \IPS\Http\Url::internal( 'app=foo...' )	// URI to link to
	'class'	=> 'modalLink'	// CSS Class to use on link (Optional)
	),
	...							// Additional buttons
	);
	 * @endcode
	 * @param	string	$url		Base URL
	 * @param	bool	$subnode	Is this a subnode?
	 * @return	array
	 */
	public function getButtons( $url, $subnode=FALSE )
	{
		$buttons = parent::getButtons( $url, $subnode );

		if( $this->converted_members )
		{
			unset( $buttons['delete'] );

			$buttons['members'] = array(
				'icon'	=> 'users',
				'title'	=> 'campaign_view_members',
				'link'	=> $url->setQueryString( array( 'subnode' => (int) isset( static::$subnodeClass ), 'do' => 'members', 'campaign' => $this->_id ) ),
				'data'	=> ( static::$modalForms ? array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack( 'campaign_campaign_members' ) ) : array() )
			);
		}

		return $buttons;
	}
}